/*
 *  Copyright 2013-2050 Emmanuel BRUN (contact@amapj.fr)
 * 
 *  This file is part of AmapJ.
 *  
 *  AmapJ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.

 *  AmapJ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with AmapJ.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 */
 package fr.amapj.view.engine.infoservlet;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import com.google.gson.Gson;

import fr.amapj.common.CollectionUtils;
import fr.amapj.model.engine.tools.SpecificDbUtils;
import fr.amapj.service.services.appinstance.AppInstanceService;
import fr.amapj.service.services.parametres.ParametresDTO;
import fr.amapj.service.services.parametres.ParametresService;
import fr.amapj.view.engine.ui.AppConfiguration;

/**
 * Servlet pour fournir des informations générales
 * 
 */
public class InfoServlet extends HttpServlet 
{
	
	private final static Logger logger = LogManager.getLogger();
	
	public enum InfoType
	{
		MONITOR , 
		LISTE_AMAP ,
		MAINTENANCE,
		BACKUP , 
		APPINSTANCE_STAT_INFO
	}
	
	@Override
	protected void service(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException 
	{
		InfoType type = getInfoType(req.getPathInfo());
		
		switch (type)
		{
		case MONITOR:
			doMonitor(res);
			break;
		
		case LISTE_AMAP:
			doListeAmap(res);
			break;

		case MAINTENANCE:
			doMaintenance(res);
			break;
			
		case BACKUP:
			doBackup(req,res);
			break;
			
		case APPINSTANCE_STAT_INFO:
			doAppInstanceStatInfo(req,res);
			break;
			
		default:
			// Do nothing 
			break;
		}	
	}







	private InfoType getInfoType(String pathInfo)
	{
		if (pathInfo==null)
		{
			return null;
		}
		
		if (pathInfo.startsWith("/monitor"))
		{
			return InfoType.MONITOR;
		}
		
		if (pathInfo.startsWith("/liste-amaps"))
		{
			return InfoType.LISTE_AMAP;
		}
		
		if (pathInfo.startsWith("/maintenance"))
		{
			return InfoType.MAINTENANCE;
		}
		
		if (pathInfo.startsWith("/backup"))
		{
			return InfoType.BACKUP;
		}
		
		if (pathInfo.startsWith("/appinstance_stat_info"))
		{
			return InfoType.APPINSTANCE_STAT_INFO;
		}
		
		return null;
		
	}

	/**
	 * Permet le monitoring des différentes valeurs du serveur
	 *  
	 * @param res
	 * @throws IOException 
	 */
	private void doMonitor(HttpServletResponse res) throws IOException
	{
		res.setContentType("application/json");
		
		MonitorInfo info =  MonitorInfo.calculateMonitorInfo();
		Gson gson = new Gson();
		
		PrintWriter out = res.getWriter();
		out.println(gson.toJson(info));
	}
	
	
	/**
	 * Permet d'afficher la liste des amaps presentes sur cette instance  
	 *  
	 * @param res
	 * @throws IOException 
	 */
	private void doListeAmap(HttpServletResponse res) throws IOException
	{
		res.setContentType("text/html");
		PrintWriter out = res.getWriter();
		
		out.write("<html><head><title></title></head><body>");
		
		out.write("<h1>Liste des AMAPs</h1>");
		out.write(getAllInstances());
		out.write("</body></html>");
	}
	
	
	
	/**
	 *  
	 * @return
	 */
	public String getAllInstances()
	{
		List<String> strs = new ArrayList<String>();
		
		SpecificDbUtils.executeInAllDb(()->appendInfos(strs),true);
		
		// On supprime la base master
		strs.remove(0);
		
		return CollectionUtils.asString(strs, "");
	}
	
	
	private Void appendInfos(List<String> strs)
	{
		ParametresDTO dto = new ParametresService().getParametres();
		String str = "<b>"+dto.nomAmap+"</b><br/>"+dto.villeAmap+"<br/><a target=\"_blank\" href=\""+dto.url+"\">"+dto.url+"</a><br/><br/><br/>";
		strs.add(str);
		return null;
	}

	
	private void doMaintenance(HttpServletResponse res) throws IOException
	{
		res.setContentType("text/html");
		PrintWriter out = res.getWriter();
		
		out.write("<html><head><title></title></head><body>");
		
		out.write("<h1>Application non disponible</h1>");
		out.write("<p>Désolé, l'application est en cours de maintenance ou votre abonnement a expiré. Merci de vous reconnecter plus tard.</p>");
		out.write("</body></html>");
		
	}
	
	
	private void doBackup(HttpServletRequest req, HttpServletResponse res) throws IOException 
	{
		logger.info("Debut du backup");
		checkMaintenanceKey(req);
		
		InputStream is = req.getInputStream();
		String backupDir = AppConfiguration.getConf().getBackupDirectory();
		String date =  readInStream(is,10);
				
		// Verification de la validité de la date 
		for (int i = 0; i < date.length(); i++) 
		{
			char c = date.charAt(i);
			if (isAllowed(c)==false)
			{
				throw new IOException("Erreur de validité de la date - c="+c);
			}
		}
		
		logger.info("Date du backup : "+date);
		
		// 
		OutputStream os = res.getOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(os);
		File[] fs = new File(backupDir+"/"+date).listFiles();
		if (fs==null)
		{
			throw new IOException("Le repertoire "+backupDir+"/"+date+" n'existe pas.");
		}
		
		oos.writeInt(fs.length);
		
		for (File file : fs) 
		{
			logger.info("Envoi du fichier : "+file.getName());
			oos.writeObject(file.getName());
			byte[] bs = FileUtils.readFileToByteArray(file);
			oos.writeObject(bs);
		}

		oos.flush();
	}


	private boolean isAllowed(char c) 
	{
		if (c=='_')
		{
			return true;
		}
		if (c>='0' && c<='9')
		{
			return true;
		}
		return false;
	}



	
	private void doAppInstanceStatInfo(HttpServletRequest req, HttpServletResponse res) throws IOException 
	{
		checkMaintenanceKey(req);
		
		// 
		OutputStream os = res.getOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(os);
		
		String str = new AppInstanceService().getStatInfo();
		oos.writeObject(str);
		oos.flush();
	}


	// Partie technique 

	private String readInStream(InputStream is , int numberOfByte) throws IOException 
	{
		byte[] bs = new byte[numberOfByte];
		IOUtils.readFully(is, bs);		
		return new String(bs,"UTF-8");
	}
	

	private void checkMaintenanceKey(HttpServletRequest req) throws IOException 
	{
		logger.info("Debut verification de la cle de maintenance");
		InputStream is = req.getInputStream();
		
		String maintenanceKey = AppConfiguration.getConf().getMaintenanceKey();
		String key = readInStream(is,40);
		
		//
		if (key.equals(maintenanceKey)==false)
		{
			logger.info("Cle de maintenance erronnée");
			throw new IOException("Erreur dans la clé de maintenance");
		}
		logger.info("Cle de maintenance correcte");
		
	}
}
